package	jp.co.sds_corp.hibiya.adapter;

import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;
import java.util.Map;

import jp.co.sds_corp.hibiya.connector.ConnectionEvent;
import jp.co.sds_corp.hibiya.connector.IConnectionListener;
import jp.co.sds_corp.hibiya.server.CheckerChangeEvent;
import jp.co.sds_corp.hibiya.server.IChecker;
import jp.co.sds_corp.hibiya.server.ICheckerListener;
import jp.co.sds_corp.util.common.CmnLog;
import jp.co.sds_corp.util.config.IConfiguration;

/**
* RMIを使用して{@link IChecker}の情報を管理します。
*/
public class RMIAdapter extends AbstractAdapter
{
	private static final long serialVersionUID = 1L;
	String		host;
	int			port;
	String		name;
	String		url;
	Registry    reg;
	transient Receiver	rcv;

	public void init( IConfiguration config ) throws Exception {
		super.init( config );
		rcv = new Receiver();
		host = config.get( "host" );
		port = Integer.valueOf( config.get( "port" ) ).intValue();
		name = config.get( "name" );
		url = "rmi://" + host + ":" + port + "/" + name;

		try {
			reg = LocateRegistry.createRegistry( port );
		} catch ( RemoteException rex ) {
			reg = LocateRegistry.getRegistry( host, port );
		}

		Naming.bind( url, rcv );
		CmnLog.debug( this, "RMIAdapter binded. url=" + url );
	}

	public void start() {
	}

	public void stop() {
		CmnLog.debug( this, "stop " + getClass().getName() );
		try {
			if ( reg != null && url != null ) {
				Naming.unbind( url );
			}
		} catch ( Exception ex ) {
			CmnLog.error( this, "unbind error", ex );
		}
		ConnectionEvent ev = new ConnectionEvent( this );
		fireDisconnected( ev );
	}

	public void saveConfig( IConfiguration config ) {
		config.addChild( "host", host );
		config.addChild( "port", String.valueOf( port ) );
		config.addChild( "name", name );
	}

	public interface IReceiver extends Remote
	{
		public void healthcheck() throws RemoteException;
		public Map<String,IChecker> getCheckerMap() throws RemoteException;
		public IChecker getChecker( String name ) throws RemoteException;
		public void addChecker( IChecker chk ) throws RemoteException;
		public void removeChecker( IChecker chk ) throws RemoteException;
		public void addCheckerListener( ICheckerListener l )
			throws Exception;
		public void removeCheckerListener( ICheckerListener l )
			throws Exception;
		public void addConnectionListener( IConnectionListener l )
			throws Exception;
		public void removeConnectionListener( IConnectionListener l )
			throws Exception;
	}

	public class Receiver extends UnicastRemoteObject implements IReceiver
	{
		private static final long serialVersionUID = 1L;

		public Receiver()
			throws RemoteException, MalformedURLException
		{
		}

		public void healthcheck() throws RemoteException {
		}

		public void addChecker( IChecker chk ) throws RemoteException {
			//RMIAdapter.this.addChecker( chk );
		}

		public void removeChecker( IChecker chk ) throws RemoteException {
			IChecker org = (IChecker )mapChk.get( chk.getName() );
			if ( org == null ) {
				return;
			}
			CheckerChangeEvent ev = new CheckerChangeEvent( org,
				org.getCheckerStatus(), IChecker.Status.DEL, null, "Deleted." );
			org.setCheckerStatus( IChecker.Status.DEL );
			try {
				org.fireCheckerChange( ev );
			} catch ( Exception e ) {
				CmnLog.error( RMIAdapter.this, "removeChecker() error", e );
			}
			RMIAdapter.this.removeChecker( chk );
		}

		public IChecker getChecker( String name ) throws RemoteException {
			if ( !isStarted()) {
				throw( new RuntimeException( "not started" ));
			}
			return (IChecker )RMIAdapter.this.getCheckerMap().get( name );
		}

		public Map<String,IChecker> getCheckerMap() throws RemoteException {
			if ( !isStarted()) {
				throw( new RuntimeException( "not started" ));
			}
			return RMIAdapter.this.getCheckerMap();
		}

		/**
		* 状態の変更を受け取るオブジェクトを追加します。
		* @param	l		状態の変更を受け取るオブジェクト
		*/
		public void addCheckerListener( ICheckerListener l ) {
			RMIAdapter.this.addCheckerListener( l );
		}

		/**
		* 状態の変更を受け取るオブジェクトを削除します。
		* @param	l		状態の変更を受け取るオブジェクト
		*/
		public void removeCheckerListener( ICheckerListener l ) {
			RMIAdapter.this.removeCheckerListener( l );
		}

		/**
		* 接続状態の変更を受け取るオブジェクトを追加します。
		* @param	l		接続状態の変更を受け取るオブジェクト
		*/
		public void addConnectionListener( IConnectionListener l ) {
			RMIAdapter.this.addConnectionListener( l );
		}

		/**
		* 接続状態の変更を受け取るオブジェクトを削除します。
		* @param	l		接続状態の変更を受け取るオブジェクト
		*/
		public void removeConnectionListener( IConnectionListener l ) {
			RMIAdapter.this.removeConnectionListener( l );
		}

	}

}

