package	jp.co.sds_corp.hibiya.server;

import java.sql.DatabaseMetaData;

import jp.co.sds_corp.util.common.CmnDate;
import jp.co.sds_corp.util.common.JarClassLoader;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.db.DbAccess;
import jp.co.sds_corp.util.db.DbInfo;
import jp.co.sds_corp.util.db.DbTable;

/**
DBサーバの監視を行います。
<BR>
XMLは以下の書式です。
<PRE>
&lt;Service name="Db1" class="jp.co.sds_corp.hibiya.server.DbChecker"&gt;
	&lt;Jar&gt;jconn2.jar&lt;/Jar&gt;
	&lt;URL&gt;jdbc:sybase:Tds:192.168.0.20:4100/DB&lt;/URL&gt;
	&lt;Driver&gt;com.sybase.jdbc2.jdbc.SybDriver&lt;/Driver&gt;
	&lt;User&gt;user&lt;/User&gt;
	&lt;Password&gt;passwd&lt;/Password&gt;
	&lt;Table&gt;test&lt;/Table&gt;
	&lt;SQL&gt;test&lt;/SQL&gt;
</PRE>
Table要素またはSQL要素を指定してください。
<p>
Action要素については、{@link jp.co.sds_corp.hibiya.action.IAction}
の説明を参照してください。
*/
public class DbChecker extends AbstractChecker
{
	private static final long serialVersionUID = 1L;
	String	jar;
	String	driver;
	String	url;
	String	user;
	String	password;
	String	table;
	String	sql;
	String	execsql;

	public DbChecker() {
		setType( "DB" );
	}

	/**
	* 設定情報で初期化します。
	* @param	config	設定情報
	*/
	public void init( IConfiguration config ) throws Exception {
		super.init( config );

		setJar( config.get( "Jar", null ));
		setDriver( config.get( "Driver", null ));
		setURL( config.get( "URL", null ));
		setUser( config.get( "User", null ));
		setPassword( config.get( "Password", null ));
		setTable( config.get( "Table", null ));
		setSql( config.get( "SQL", null ));
	}

	/**
	* 監視処理を実行します。
	*/
	public void check() {
		DbAccess db = null;
		try {
			CmnDate startTime = new CmnDate();
			db = connect();
			DbTable tbl = execSql( db );
			db.rollback();
			CmnDate endTime = new CmnDate();
			clearResult();
			setResponseTime( startTime, endTime );

			if ( getSql() == null || getSql().length() == 0 ) {
				putResult( "Count", tbl.getObject( 0 ));
			} else {
				for ( int i = 0; i < tbl.getColumnCount(); i++ ) {
					putResult( tbl.getColumn( i ).getName(), tbl.getObject( i ));
				}
			}
			getInformation( db );

			setCheckerStatus( IChecker.Status.OK );
		} catch ( Throwable ex ) {
			synchronized( this ) {
				clearResult();
				setException( ex );
				setCheckerStatus( IChecker.Status.NG );
			}
		} finally {
			try {
				db.disconnect();
			} catch ( Exception ex ) {
			}
		}
	}

	String prevJar = null;
	transient JarClassLoader loader;
	/**
	* DBサーバと接続します。
	*/
	protected DbAccess connect() throws Exception {
		DbInfo info = new DbInfo( getDriver(), getURL(), getUser(), getPassword(), null );
		DbAccess db = new DbAccess( info );
		String jar = getJar();
		if ( jar != null && jar.length() > 0 ) {
			if ( !jar.equals( prevJar )) {
				loader = new JarClassLoader( jar, getClass().getClassLoader() );
				prevJar = jar;
			}
			Class<?> cl = loader.loadClass( getDriver() );
			java.sql.Driver drv = (java.sql.Driver )cl.newInstance();
			db.connect( drv );
		} else {
			db.connect();
		}
		return db;
	}

	/**
	* SQLを実行します。
	* @param	db		DB接続情報
	*/
	protected DbTable execSql( DbAccess db ) throws Exception {
		db.executeQuery( execsql );
		DbTable tbl = new DbTable();
		for ( ;; ) {
			if ( !db.fetch( tbl )) {
				break;
			}
		}
		db.cancel();
		return tbl;
	}

	/**
	* DBサーバの情報を取得し、結果にセットします。
	*/
	private void getInformation( DbAccess db ) throws Exception {
		DatabaseMetaData meta = db.getConnection().getMetaData();
		String var = null;
		try {
			var = meta.getDatabaseProductName();
		} catch ( Throwable e ) {}
		putResult( "Database", var );

		var = null;
		try {
			var = meta.getDatabaseMajorVersion()
					+ "." + meta.getDatabaseMinorVersion();
		} catch ( Throwable e ) {}
		putResult( "DatabaseVersion", var );

		var = null;
		try {
			var = meta.getDriverName();
		} catch ( Throwable e ) {}
		putResult( "Driver", var );

		var = null;
		try {
			var = meta.getDriverMajorVersion()
					+ "." + meta.getDriverMinorVersion();
		} catch ( Throwable e ) {}
		putResult( "DriverVersion", var );

		var = null;
		try {
			var = meta.getJDBCMajorVersion()
					+ " " + meta.getJDBCMinorVersion();
		} catch ( Throwable e ) {}
		putResult( "JDBCVersion", var );
	}

	/**
	* JDBCドライバのjar(jar)を設定します。
	* @param	var		設定するデータ
	*/
	public void setJar( String var ) {
		this.jar = var ;
	}
	/**
	* JDBCドライバのjar(jar)を取得します。
	* @return	jarの内容
	*/
	public String getJar() {
		return jar;
	}

	/**
	* JDBCドライバ名(driver)を設定します。
	* @param	var		設定するデータ
	*/
	public void setDriver( String var ) {
		this.driver = var ;
	}
	/**
	* JDBCドライバ名(driver)を取得します。
	* @return	driverの内容
	*/
	public String getDriver() {
		return driver;
	}

	/**
	* JDBCドライバのURL(url)を設定します。
	* @param	var		設定するデータ
	*/
	public void setURL( String var ) {
		this.url = var ;
	}
	/**
	* JDBCドライバのURL(url)を取得します。
	* @return	urlの内容
	*/
	public String getURL() {
		return url;
	}

	/**
	* JDBCドライバのユーザ(user)を設定します。
	* @param	var		設定するデータ
	*/
	public void setUser( String var ) {
		this.user = var ;
	}
	/**
	* JDBCドライバのユーザ(user)を取得します。
	* @return	userの内容
	*/
	public String getUser() {
		return user;
	}

	/**
	* JDBCドライバのパスワード(password)を設定します。
	* @param	var		設定するデータ
	*/
	public void setPassword( String var ) {
		this.password = var ;
	}
	/**
	* JDBCドライバのパスワード(password)を取得します。
	* @return	passwordの内容
	*/
	public String getPassword() {
		return password;
	}

	/**
	* チェックに使用するテーブル名(table)を設定します。
	* @param	var		設定するデータ
	*/
	public void setTable( String var ) {
		this.table = var ;
		if ( var != null && var.length() > 0 ) {
			execsql = "select count(*) from " + var;
		}
	}
	/**
	* チェックに使用するテーブル名(table)を取得します。
	* @return	tableの内容
	*/
	public String getTable() {
		return table;
	}

	/**
	* チェックに使用するSQLを設定します。
	* @param	var		設定するデータ
	*/
	public void setSql( String var ) {
		this.sql = var ;
		if ( var != null && var.length() > 0 ) {
			execsql = var;
		}
	}
	/**
	* チェックに使用するSQLを取得します。
	* @return	tableの内容
	*/
	public String getSql() {
		return sql;
	}

	/**
	* 設定されている内容をXML形式で出力します。
	* @param	config	設定情報
	*/
	public void saveConfig( IConfiguration config ) {
		super.saveConfig( config );
		if ( jar != null ) {
			config.addChild( "Jar", jar );
		}
		config.addChild( "Driver", driver );
		config.addChild( "URL", url );
		if ( user != null ) {
			config.addChild( "User", user );
		}
		if ( password != null ) {
			config.addChild( "Password", password );
		}
		if ( table != null ) {
			config.addChild( "Table", table );
		}
		if ( sql != null ) {
			config.addChild( "SQL", sql );
		}
	}

	/**
	* このモニタの文字列表現を返します。
	* @return	このモニタの文字列表現
	*/
	public String toString() {
		String ret = super.toString();
		ret += "["
			+ "Jar=" + jar + ";"
			+ "Driver=" + driver + ";"
			+ "URL=" + url + ";"
			+ "User=" + user + ";"
//			+ "Password=" + password + ";"
			+ "Password=*;"
			+ "Table=" + table + ";"
			+ "Sql=" + sql + ";"
			+ "]";
		return ret;
	}

}
